#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from unittest import mock

from osc_lib import exceptions

from openstackclient.api import compute_v2
from openstackclient.network import utils as network_utils
from openstackclient.network.v2 import security_group_rule
from openstackclient.tests.unit.compute.v2 import fakes as compute_fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
from openstackclient.tests.unit import utils as tests_utils


@mock.patch.object(compute_v2, 'create_security_group_rule')
class TestCreateSecurityGroupRuleCompute(compute_fakes.TestComputev2):
    project = identity_fakes.FakeProject.create_one_project()
    domain = identity_fakes.FakeDomain.create_one_domain()

    # The security group rule to be created.
    _security_group_rule = None

    # The security group that will contain the rule created.
    _security_group = compute_fakes.create_one_security_group()

    def _setup_security_group_rule(self, attrs=None):
        self._security_group_rule = (
            compute_fakes.create_one_security_group_rule(attrs)
        )
        (
            expected_columns,
            expected_data,
        ) = network_utils.format_security_group_rule_show(
            self._security_group_rule
        )
        return expected_columns, expected_data

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        compute_v2.find_security_group = mock.Mock(
            return_value=self._security_group,
        )

        # Get the command object to test
        self.cmd = security_group_rule.CreateSecurityGroupRule(self.app, None)

    def test_security_group_rule_create_no_options(self, sgr_mock):
        self.assertRaises(
            tests_utils.ParserException, self.check_parser, self.cmd, [], []
        )

    def test_security_group_rule_create_all_remote_options(self, sgr_mock):
        arglist = [
            '--remote-ip',
            '10.10.0.0/24',
            '--remote-group',
            self._security_group['id'],
            self._security_group['id'],
        ]
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            [],
        )

    def test_security_group_rule_create_bad_protocol(self, sgr_mock):
        arglist = [
            '--protocol',
            'foo',
            self._security_group['id'],
        ]
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            [],
        )

    def test_security_group_rule_create_all_protocol_options(self, sgr_mock):
        arglist = [
            '--protocol',
            'tcp',
            '--proto',
            'tcp',
            self._security_group['id'],
        ]
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            [],
        )

    def test_security_group_rule_create_network_options(self, sgr_mock):
        arglist = [
            '--ingress',
            '--ethertype',
            'IPv4',
            '--icmp-type',
            '3',
            '--icmp-code',
            '11',
            '--project',
            self.project.name,
            '--project-domain',
            self.domain.name,
            self._security_group['id'],
        ]
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            [],
        )

    def test_security_group_rule_create_default_rule(self, sgr_mock):
        expected_columns, expected_data = self._setup_security_group_rule()
        sgr_mock.return_value = self._security_group_rule
        dst_port = (
            str(self._security_group_rule['from_port'])
            + ':'
            + str(self._security_group_rule['to_port'])
        )
        arglist = [
            '--dst-port',
            dst_port,
            self._security_group['id'],
        ]
        verifylist = [
            (
                'dst_port',
                (
                    self._security_group_rule['from_port'],
                    self._security_group_rule['to_port'],
                ),
            ),
            ('group', self._security_group['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sgr_mock.assert_called_once_with(
            self.compute_client,
            security_group_id=self._security_group['id'],
            ip_protocol=self._security_group_rule['ip_protocol'],
            from_port=self._security_group_rule['from_port'],
            to_port=self._security_group_rule['to_port'],
            remote_ip=self._security_group_rule['ip_range']['cidr'],
            remote_group=None,
        )
        self.assertEqual(expected_columns, columns)
        self.assertEqual(expected_data, data)

    def test_security_group_rule_create_remote_group(self, sgr_mock):
        expected_columns, expected_data = self._setup_security_group_rule(
            {
                'from_port': 22,
                'to_port': 22,
                'group': {'name': self._security_group['name']},
            }
        )
        sgr_mock.return_value = self._security_group_rule
        arglist = [
            '--dst-port',
            str(self._security_group_rule['from_port']),
            '--remote-group',
            self._security_group['name'],
            self._security_group['id'],
        ]
        verifylist = [
            (
                'dst_port',
                (
                    self._security_group_rule['from_port'],
                    self._security_group_rule['to_port'],
                ),
            ),
            ('remote_group', self._security_group['name']),
            ('group', self._security_group['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sgr_mock.assert_called_once_with(
            self.compute_client,
            security_group_id=self._security_group['id'],
            ip_protocol=self._security_group_rule['ip_protocol'],
            from_port=self._security_group_rule['from_port'],
            to_port=self._security_group_rule['to_port'],
            remote_ip=self._security_group_rule['ip_range']['cidr'],
            remote_group=self._security_group['id'],
        )
        self.assertEqual(expected_columns, columns)
        self.assertEqual(expected_data, data)

    def test_security_group_rule_create_remote_ip(self, sgr_mock):
        expected_columns, expected_data = self._setup_security_group_rule(
            {
                'ip_protocol': 'icmp',
                'from_port': -1,
                'to_port': -1,
                'ip_range': {'cidr': '10.0.2.0/24'},
            }
        )
        sgr_mock.return_value = self._security_group_rule
        arglist = [
            '--protocol',
            self._security_group_rule['ip_protocol'],
            '--remote-ip',
            self._security_group_rule['ip_range']['cidr'],
            self._security_group['id'],
        ]
        verifylist = [
            ('protocol', self._security_group_rule['ip_protocol']),
            ('remote_ip', self._security_group_rule['ip_range']['cidr']),
            ('group', self._security_group['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sgr_mock.assert_called_once_with(
            self.compute_client,
            security_group_id=self._security_group['id'],
            ip_protocol=self._security_group_rule['ip_protocol'],
            from_port=self._security_group_rule['from_port'],
            to_port=self._security_group_rule['to_port'],
            remote_ip=self._security_group_rule['ip_range']['cidr'],
            remote_group=None,
        )
        self.assertEqual(expected_columns, columns)
        self.assertEqual(expected_data, data)

    def test_security_group_rule_create_proto_option(self, sgr_mock):
        expected_columns, expected_data = self._setup_security_group_rule(
            {
                'ip_protocol': 'icmp',
                'from_port': -1,
                'to_port': -1,
                'ip_range': {'cidr': '10.0.2.0/24'},
            }
        )
        sgr_mock.return_value = self._security_group_rule
        arglist = [
            '--proto',
            self._security_group_rule['ip_protocol'],
            '--remote-ip',
            self._security_group_rule['ip_range']['cidr'],
            self._security_group['id'],
        ]
        verifylist = [
            ('proto', self._security_group_rule['ip_protocol']),
            ('protocol', None),
            ('remote_ip', self._security_group_rule['ip_range']['cidr']),
            ('group', self._security_group['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sgr_mock.assert_called_once_with(
            self.compute_client,
            security_group_id=self._security_group['id'],
            ip_protocol=self._security_group_rule['ip_protocol'],
            from_port=self._security_group_rule['from_port'],
            to_port=self._security_group_rule['to_port'],
            remote_ip=self._security_group_rule['ip_range']['cidr'],
            remote_group=None,
        )
        self.assertEqual(expected_columns, columns)
        self.assertEqual(expected_data, data)


@mock.patch.object(compute_v2, 'delete_security_group_rule')
class TestDeleteSecurityGroupRuleCompute(compute_fakes.TestComputev2):
    # The security group rule to be deleted.
    _security_group_rules = compute_fakes.create_security_group_rules(count=2)

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        # Get the command object to test
        self.cmd = security_group_rule.DeleteSecurityGroupRule(self.app, None)

    def test_security_group_rule_delete(self, sgr_mock):
        arglist = [
            self._security_group_rules[0]['id'],
        ]
        verifylist = [
            ('rule', [self._security_group_rules[0]['id']]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        sgr_mock.assert_called_once_with(
            self.compute_client, self._security_group_rules[0]['id']
        )
        self.assertIsNone(result)

    def test_security_group_rule_delete_multi(self, sgr_mock):
        arglist = [
            self._security_group_rules[0]['id'],
            self._security_group_rules[1]['id'],
        ]
        verifylist = [
            ('rule', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        sgr_mock.assert_has_calls(
            [
                mock.call(
                    self.compute_client,
                    self._security_group_rules[0]['id'],
                ),
                mock.call(
                    self.compute_client,
                    self._security_group_rules[1]['id'],
                ),
            ]
        )
        self.assertIsNone(result)

    def test_security_group_rule_delete_multi_with_exception(self, sgr_mock):
        arglist = [
            self._security_group_rules[0]['id'],
            'unexist_rule',
        ]
        verifylist = [
            ('rule', arglist),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        sgr_mock.side_effect = [None, exceptions.NotFound('foo')]

        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual('1 of 2 rules failed to delete.', str(e))

        sgr_mock.assert_has_calls(
            [
                mock.call(
                    self.compute_client,
                    self._security_group_rules[0]['id'],
                ),
                mock.call(self.compute_client, 'unexist_rule'),
            ]
        )


class TestListSecurityGroupRuleCompute(compute_fakes.TestComputev2):
    # The security group to hold the rules.
    _security_group = compute_fakes.create_one_security_group()

    # The security group rule to be listed.
    _security_group_rule_tcp = compute_fakes.create_one_security_group_rule(
        {
            'ip_protocol': 'tcp',
            'from_port': 80,
            'to_port': 80,
            'group': {'name': _security_group['name']},
        }
    )
    _security_group_rule_icmp = compute_fakes.create_one_security_group_rule(
        {
            'ip_protocol': 'icmp',
            'from_port': -1,
            'to_port': -1,
            'ip_range': {'cidr': '10.0.2.0/24'},
            'group': {'name': _security_group['name']},
        }
    )
    _security_group['rules'] = [
        _security_group_rule_tcp,
        _security_group_rule_icmp,
    ]

    expected_columns_with_group = (
        'ID',
        'IP Protocol',
        'Ethertype',
        'IP Range',
        'Port Range',
        'Direction',
        'Remote Security Group',
    )
    expected_columns_no_group = expected_columns_with_group + (
        'Security Group',
    )

    expected_data_with_group = []
    expected_data_no_group = []
    for _security_group_rule in _security_group['rules']:
        rule = network_utils.transform_compute_security_group_rule(
            _security_group_rule
        )
        expected_rule_with_group = (
            rule['id'],
            rule['ip_protocol'],
            '',  # ethertype is a neutron-only thing
            rule['ip_range'],
            rule['port_range'],
            rule['remote_security_group'],
        )
        expected_rule_no_group = expected_rule_with_group + (
            _security_group_rule['parent_group_id'],
        )
        expected_data_with_group.append(expected_rule_with_group)
        expected_data_no_group.append(expected_rule_no_group)

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        compute_v2.find_security_group = mock.Mock(
            return_value=self._security_group,
        )
        compute_v2.list_security_groups = mock.Mock(
            return_value=[self._security_group],
        )

        # Get the command object to test
        self.cmd = security_group_rule.ListSecurityGroupRule(self.app, None)

    def test_security_group_rule_list_default(self):
        parsed_args = self.check_parser(self.cmd, [], [])

        columns, data = self.cmd.take_action(parsed_args)
        compute_v2.list_security_groups.assert_called_once_with(
            self.compute_client, all_projects=False
        )
        self.assertEqual(self.expected_columns_no_group, columns)
        self.assertEqual(self.expected_data_no_group, list(data))

    def test_security_group_rule_list_with_group(self):
        arglist = [
            self._security_group['id'],
        ]
        verifylist = [
            ('group', self._security_group['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        compute_v2.find_security_group.assert_called_once_with(
            self.compute_client, self._security_group['id']
        )
        self.assertEqual(self.expected_columns_with_group, columns)
        self.assertEqual(self.expected_data_with_group, list(data))

    def test_security_group_rule_list_all_projects(self):
        arglist = [
            '--all-projects',
        ]
        verifylist = [
            ('all_projects', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        compute_v2.list_security_groups.assert_called_once_with(
            self.compute_client, all_projects=True
        )
        self.assertEqual(self.expected_columns_no_group, columns)
        self.assertEqual(self.expected_data_no_group, list(data))

    def test_security_group_rule_list_with_ignored_options(self):
        arglist = [
            '--long',
        ]
        verifylist = [
            ('long', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        compute_v2.list_security_groups.assert_called_once_with(
            self.compute_client, all_projects=False
        )
        self.assertEqual(self.expected_columns_no_group, columns)
        self.assertEqual(self.expected_data_no_group, list(data))


class TestShowSecurityGroupRuleCompute(compute_fakes.TestComputev2):
    # The security group rule to be shown.
    _security_group_rule = compute_fakes.create_one_security_group_rule()

    columns, data = network_utils.format_security_group_rule_show(
        _security_group_rule
    )

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        # Build a security group fake customized for this test.
        security_group_rules = [self._security_group_rule]
        security_group = {'rules': security_group_rules}
        compute_v2.list_security_groups = mock.Mock(
            return_value=[security_group],
        )

        # Get the command object to test
        self.cmd = security_group_rule.ShowSecurityGroupRule(self.app, None)

    def test_security_group_rule_show_no_options(self):
        self.assertRaises(
            tests_utils.ParserException, self.check_parser, self.cmd, [], []
        )

    def test_security_group_rule_show_all_options(self):
        arglist = [
            self._security_group_rule['id'],
        ]
        verifylist = [
            ('rule', self._security_group_rule['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        compute_v2.list_security_groups.assert_called_once_with(
            self.compute_client
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Network segment action implementations"""

import logging

from osc_lib import exceptions
from osc_lib import utils

from openstackclient import command
from openstackclient.i18n import _
from openstackclient.network import common

LOG = logging.getLogger(__name__)


def _get_columns(item):
    hidden_columns = ['location', 'tenant_id']
    return utils.get_osc_show_columns_for_sdk_resource(
        item, {}, hidden_columns
    )


class CreateNetworkSegment(
    command.ShowOne, common.NeutronCommandWithExtraArgs
):
    _description = _("Create new network segment")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'name', metavar='<name>', help=_('New network segment name')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Network segment description'),
        )
        parser.add_argument(
            '--physical-network',
            metavar='<physical-network>',
            help=_('Physical network name of this network segment'),
        )
        parser.add_argument(
            '--segment',
            metavar='<segment>',
            type=int,
            help=_(
                'Segment identifier for this network segment which is '
                'based on the network type, VLAN ID for vlan network '
                'type and tunnel ID for geneve, gre and vxlan network '
                'types'
            ),
        )
        parser.add_argument(
            '--network',
            metavar='<network>',
            required=True,
            help=_('Network this network segment belongs to (name or ID)'),
        )
        parser.add_argument(
            '--network-type',
            metavar='<network-type>',
            choices=['flat', 'geneve', 'gre', 'local', 'vlan', 'vxlan'],
            required=True,
            help=_(
                'Network type of this network segment '
                '(flat, geneve, gre, local, vlan or vxlan)'
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        attrs = {}
        attrs['name'] = parsed_args.name
        attrs['network_id'] = client.find_network(
            parsed_args.network, ignore_missing=False
        ).id
        attrs['network_type'] = parsed_args.network_type
        if parsed_args.description is not None:
            attrs['description'] = parsed_args.description
        if parsed_args.physical_network is not None:
            attrs['physical_network'] = parsed_args.physical_network
        if parsed_args.segment is not None:
            attrs['segmentation_id'] = parsed_args.segment
        attrs.update(
            self._parse_extra_properties(parsed_args.extra_properties)
        )
        obj = client.create_segment(**attrs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)


class DeleteNetworkSegment(command.Command):
    _description = _("Delete network segment(s)")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'network_segment',
            metavar='<network-segment>',
            nargs='+',
            help=_('Network segment(s) to delete (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network

        result = 0
        for network_segment in parsed_args.network_segment:
            try:
                obj = client.find_segment(
                    network_segment, ignore_missing=False
                )
                client.delete_segment(obj)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        "Failed to delete network segment with "
                        "ID '%(network_segment)s': %(e)s"
                    ),
                    {'network_segment': network_segment, 'e': e},
                )

        if result > 0:
            total = len(parsed_args.network_segment)
            msg = _(
                "%(result)s of %(total)s network segments failed to delete."
            ) % {'result': result, 'total': total}
            raise exceptions.CommandError(msg)


class ListNetworkSegment(command.Lister):
    _description = _("List network segments")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_('List additional fields in output'),
        )
        parser.add_argument(
            '--network',
            metavar='<network>',
            help=_(
                'List only network segments associated with the specified '
                'network (name or ID)'
            ),
        )
        return parser

    def take_action(self, parsed_args):
        network_client = self.app.client_manager.network

        filters = {}
        if parsed_args.network:
            _network = network_client.find_network(
                parsed_args.network, ignore_missing=False
            )
            filters = {'network_id': _network.id}
        data = network_client.segments(**filters)

        headers: tuple[str, ...] = (
            'ID',
            'Name',
            'Network',
            'Network Type',
            'Segment',
        )
        columns: tuple[str, ...] = (
            'id',
            'name',
            'network_id',
            'network_type',
            'segmentation_id',
        )
        if parsed_args.long:
            headers += ('Physical Network',)
            columns += ('physical_network',)

        return (
            headers,
            (
                utils.get_item_properties(
                    s,
                    columns,
                    formatters={},
                )
                for s in data
            ),
        )


class SetNetworkSegment(common.NeutronCommandWithExtraArgs):
    _description = _("Set network segment properties")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'network_segment',
            metavar='<network-segment>',
            help=_('Network segment to modify (name or ID)'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Set network segment description'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Set network segment name'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj = client.find_segment(
            parsed_args.network_segment, ignore_missing=False
        )
        attrs = {}
        if parsed_args.description is not None:
            attrs['description'] = parsed_args.description
        if parsed_args.name is not None:
            attrs['name'] = parsed_args.name
        attrs.update(
            self._parse_extra_properties(parsed_args.extra_properties)
        )
        client.update_segment(obj, **attrs)


class ShowNetworkSegment(command.ShowOne):
    _description = _("Display network segment details")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'network_segment',
            metavar='<network-segment>',
            help=_('Network segment to display (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj = client.find_segment(
            parsed_args.network_segment, ignore_missing=False
        )
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock

from osc_lib import exceptions

from openstackclient.api import compute_v2
from openstackclient.network.v2 import security_group
from openstackclient.tests.unit.compute.v2 import fakes as compute_fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
from openstackclient.tests.unit import utils as tests_utils


@mock.patch.object(compute_v2, 'create_security_group')
class TestCreateSecurityGroupCompute(compute_fakes.TestComputev2):
    project = identity_fakes.FakeProject.create_one_project()
    domain = identity_fakes.FakeDomain.create_one_domain()

    # The security group to be shown.
    _security_group = compute_fakes.create_one_security_group()

    columns = (
        'description',
        'id',
        'name',
        'project_id',
        'rules',
    )

    data = (
        _security_group['description'],
        _security_group['id'],
        _security_group['name'],
        _security_group['tenant_id'],
        security_group.ComputeSecurityGroupRulesColumn([]),
    )

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        # Get the command object to test
        self.cmd = security_group.CreateSecurityGroup(self.app, None)

    def test_security_group_create_no_options(self, sg_mock):
        self.assertRaises(
            tests_utils.ParserException, self.check_parser, self.cmd, [], []
        )

    def test_security_group_create_min_options(self, sg_mock):
        sg_mock.return_value = self._security_group
        arglist = [
            self._security_group['name'],
        ]
        verifylist = [
            ('name', self._security_group['name']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(
            self.compute_client,
            self._security_group['name'],
            self._security_group['name'],
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_security_group_create_all_options(self, sg_mock):
        sg_mock.return_value = self._security_group
        arglist = [
            '--description',
            self._security_group['description'],
            self._security_group['name'],
        ]
        verifylist = [
            ('description', self._security_group['description']),
            ('name', self._security_group['name']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(
            self.compute_client,
            self._security_group['name'],
            self._security_group['description'],
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


@mock.patch.object(compute_v2, 'delete_security_group')
class TestDeleteSecurityGroupCompute(compute_fakes.TestComputev2):
    # The security groups to be deleted.
    _security_groups = compute_fakes.create_security_groups()

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        compute_v2.find_security_group = mock.Mock(
            side_effect=self._security_groups
        )

        # Get the command object to test
        self.cmd = security_group.DeleteSecurityGroup(self.app, None)

    def test_security_group_delete(self, sg_mock):
        sg_mock.return_value = mock.Mock(return_value=None)
        arglist = [
            self._security_groups[0]['id'],
        ]
        verifylist = [
            ('group', [self._security_groups[0]['id']]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(
            self.compute_client,
            self._security_groups[0]['id'],
        )
        self.assertIsNone(result)

    def test_security_group_multi_delete(self, sg_mock):
        sg_mock.return_value = mock.Mock(return_value=None)
        arglist = [
            self._security_groups[0]['id'],
            self._security_groups[1]['id'],
        ]
        verifylist = [
            ('group', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        sg_mock.assert_has_calls(
            [
                mock.call(self.compute_client, self._security_groups[0]['id']),
                mock.call(self.compute_client, self._security_groups[1]['id']),
            ]
        )
        self.assertIsNone(result)

    def test_security_group_multi_delete_with_exception(self, sg_mock):
        sg_mock.return_value = mock.Mock(return_value=None)
        compute_v2.find_security_group.side_effect = [
            self._security_groups[0],
            exceptions.NotFound('foo'),
        ]
        arglist = [
            self._security_groups[0]['id'],
            'unexist_security_group',
        ]
        verifylist = [
            ('group', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        exc = self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args,
        )
        self.assertEqual('1 of 2 groups failed to delete.', str(exc))

        sg_mock.assert_has_calls(
            [
                mock.call(self.compute_client, self._security_groups[0]['id']),
            ]
        )


@mock.patch.object(compute_v2, 'list_security_groups')
class TestListSecurityGroupCompute(compute_fakes.TestComputev2):
    # The security group to be listed.
    _security_groups = compute_fakes.create_security_groups(count=3)

    columns = (
        'ID',
        'Name',
        'Description',
    )
    columns_all_projects = (
        'ID',
        'Name',
        'Description',
        'Project',
    )

    data = []
    for grp in _security_groups:
        data.append(
            (
                grp['id'],
                grp['name'],
                grp['description'],
            )
        )
    data_all_projects = []
    for grp in _security_groups:
        data_all_projects.append(
            (
                grp['id'],
                grp['name'],
                grp['description'],
                grp['tenant_id'],
            )
        )

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        # Get the command object to test
        self.cmd = security_group.ListSecurityGroup(self.app, None)

    def test_security_group_list_no_options(self, sg_mock):
        sg_mock.return_value = self._security_groups
        arglist = []
        verifylist = [
            ('all_projects', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(
            self.compute_client, all_projects=False
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, list(data))

    def test_security_group_list_all_projects(self, sg_mock):
        sg_mock.return_value = self._security_groups
        arglist = [
            '--all-projects',
        ]
        verifylist = [
            ('all_projects', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(self.compute_client, all_projects=True)
        self.assertEqual(self.columns_all_projects, columns)
        self.assertCountEqual(self.data_all_projects, list(data))


@mock.patch.object(compute_v2, 'update_security_group')
class TestSetSecurityGroupCompute(compute_fakes.TestComputev2):
    # The security group to be set.
    _security_group = compute_fakes.create_one_security_group()

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        compute_v2.find_security_group = mock.Mock(
            return_value=self._security_group
        )

        # Get the command object to test
        self.cmd = security_group.SetSecurityGroup(self.app, None)

    def test_security_group_set_no_options(self, sg_mock):
        self.assertRaises(
            tests_utils.ParserException, self.check_parser, self.cmd, [], []
        )

    def test_security_group_set_no_updates(self, sg_mock):
        sg_mock.return_value = mock.Mock(return_value=None)
        arglist = [
            self._security_group['name'],
        ]
        verifylist = [
            ('group', self._security_group['name']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(
            self.compute_client, self._security_group['id']
        )
        self.assertIsNone(result)

    def test_security_group_set_all_options(self, sg_mock):
        sg_mock.return_value = mock.Mock(return_value=None)
        new_name = 'new-' + self._security_group['name']
        new_description = 'new-' + self._security_group['description']
        arglist = [
            '--name',
            new_name,
            '--description',
            new_description,
            self._security_group['name'],
        ]
        verifylist = [
            ('description', new_description),
            ('group', self._security_group['name']),
            ('name', new_name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(
            self.compute_client,
            self._security_group['id'],
            name=new_name,
            description=new_description,
        )
        self.assertIsNone(result)


@mock.patch.object(compute_v2, 'find_security_group')
class TestShowSecurityGroupCompute(compute_fakes.TestComputev2):
    # The security group rule to be shown with the group.
    _security_group_rule = compute_fakes.create_one_security_group_rule()

    # The security group to be shown.
    _security_group = compute_fakes.create_one_security_group(
        attrs={'rules': [_security_group_rule]}
    )

    columns = (
        'description',
        'id',
        'name',
        'project_id',
        'rules',
    )

    data = (
        _security_group['description'],
        _security_group['id'],
        _security_group['name'],
        _security_group['tenant_id'],
        security_group.ComputeSecurityGroupRulesColumn([_security_group_rule]),
    )

    def setUp(self):
        super().setUp()

        self.app.client_manager.network_endpoint_enabled = False

        # Get the command object to test
        self.cmd = security_group.ShowSecurityGroup(self.app, None)

    def test_security_group_show_no_options(self, sg_mock):
        self.assertRaises(
            tests_utils.ParserException, self.check_parser, self.cmd, [], []
        )

    def test_security_group_show_all_options(self, sg_mock):
        sg_mock.return_value = self._security_group
        arglist = [
            self._security_group['id'],
        ]
        verifylist = [
            ('group', self._security_group['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        sg_mock.assert_called_once_with(
            self.compute_client, self._security_group['id']
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from openstackclient.image.v2 import metadef_namespaces
from openstackclient.tests.unit.image.v2 import fakes as image_fakes


class TestMetadefNamespaceCreate(image_fakes.TestImagev2):
    _metadef_namespace = image_fakes.create_one_metadef_namespace()

    expected_columns = (
        'created_at',
        'display_name',
        'namespace',
        'owner',
        'visibility',
    )
    expected_data = (
        _metadef_namespace.created_at,
        _metadef_namespace.display_name,
        _metadef_namespace.namespace,
        _metadef_namespace.owner,
        _metadef_namespace.visibility,
    )

    def setUp(self):
        super().setUp()

        self.image_client.create_metadef_namespace.return_value = (
            self._metadef_namespace
        )
        self.cmd = metadef_namespaces.CreateMetadefNamespace(self.app, None)
        self.datalist = self._metadef_namespace

    def test_namespace_create(self):
        arglist = [self._metadef_namespace.namespace]

        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(self.expected_columns, columns)
        self.assertEqual(self.expected_data, data)


class TestMetadefNamespaceDelete(image_fakes.TestImagev2):
    _metadef_namespace = image_fakes.create_one_metadef_namespace()

    def setUp(self):
        super().setUp()

        self.image_client.delete_metadef_namespace.return_value = (
            self._metadef_namespace
        )
        self.cmd = metadef_namespaces.DeleteMetadefNamespace(self.app, None)
        self.datalist = self._metadef_namespace

    def test_namespace_create(self):
        arglist = [self._metadef_namespace.namespace]

        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)


class TestMetadefNamespaceList(image_fakes.TestImagev2):
    _metadef_namespace = [image_fakes.create_one_metadef_namespace()]

    columns = ['namespace']

    datalist = []

    def setUp(self):
        super().setUp()

        self.image_client.metadef_namespaces.side_effect = [
            self._metadef_namespace,
            [],
        ]

        # Get the command object to test
        self.image_client.metadef_namespaces.return_value = iter(
            self._metadef_namespace
        )
        self.cmd = metadef_namespaces.ListMetadefNamespace(self.app, None)
        self.datalist = self._metadef_namespace

    def test_namespace_list_no_options(self):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(self.columns, columns)
        self.assertEqual(getattr(self.datalist[0], 'namespace'), next(data)[0])


class TestMetadefNamespaceSet(image_fakes.TestImagev2):
    _metadef_namespace = image_fakes.create_one_metadef_namespace()

    def setUp(self):
        super().setUp()

        self.image_client.update_metadef_namespace.return_value = (
            self._metadef_namespace
        )
        self.cmd = metadef_namespaces.SetMetadefNamespace(self.app, None)
        self.datalist = self._metadef_namespace

    def test_namespace_set_no_options(self):
        arglist = [self._metadef_namespace.namespace]
        verifylist = [
            ('namespace', self._metadef_namespace.namespace),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)


class TestMetadefNamespaceShow(image_fakes.TestImagev2):
    _metadef_namespace = image_fakes.create_one_metadef_namespace()

    expected_columns = (
        'created_at',
        'display_name',
        'namespace',
        'owner',
        'visibility',
    )
    expected_data = (
        _metadef_namespace.created_at,
        _metadef_namespace.display_name,
        _metadef_namespace.namespace,
        _metadef_namespace.owner,
        _metadef_namespace.visibility,
    )

    def setUp(self):
        super().setUp()

        self.image_client.get_metadef_namespace.return_value = (
            self._metadef_namespace
        )
        self.cmd = metadef_namespaces.ShowMetadefNamespace(self.app, None)

    def test_namespace_show_no_options(self):
        arglist = [self._metadef_namespace.namespace]

        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(self.expected_columns, columns)
        self.assertEqual(self.expected_data, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Volume v3 API Library Tests"""

import http
from unittest import mock
import uuid

from openstack.block_storage.v3 import _proxy
from osc_lib import exceptions as osc_lib_exceptions

from openstackclient.api import volume_v3 as volume
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit import utils


class TestConsistencyGroup(utils.TestCase):
    def setUp(self):
        super().setUp()

        self.volume_sdk_client = mock.Mock(_proxy.Proxy)

    def test_find_consistency_group_by_id(self):
        cg_id = uuid.uuid4().hex
        cg_name = 'name-' + uuid.uuid4().hex
        data = {
            'consistencygroup': {
                'id': cg_id,
                'name': cg_name,
                'status': 'available',
                'availability_zone': 'az1',
                'created_at': '2015-09-16T09:28:52.000000',
                'description': 'description-' + uuid.uuid4().hex,
                'volume_types': ['123456'],
            }
        }
        self.volume_sdk_client.get.side_effect = [
            fakes.FakeResponse(data=data),
        ]

        result = volume.find_consistency_group(self.volume_sdk_client, cg_id)

        self.volume_sdk_client.get.assert_has_calls(
            [
                mock.call(f'/consistencygroups/{cg_id}'),
            ]
        )
        self.assertEqual(data['consistencygroup'], result)

    def test_find_consistency_group_by_name(self):
        cg_id = uuid.uuid4().hex
        cg_name = 'name-' + uuid.uuid4().hex
        data = {
            'consistencygroups': [
                {
                    'id': cg_id,
                    'name': cg_name,
                }
            ],
        }
        self.volume_sdk_client.get.side_effect = [
            fakes.FakeResponse(status_code=http.HTTPStatus.NOT_FOUND),
            fakes.FakeResponse(data=data),
        ]

        result = volume.find_consistency_group(self.volume_sdk_client, cg_name)

        self.volume_sdk_client.get.assert_has_calls(
            [
                mock.call(f'/consistencygroups/{cg_name}'),
                mock.call('/consistencygroups'),
            ]
        )
        self.assertEqual(data['consistencygroups'][0], result)

    def test_find_consistency_group_not_found(self):
        data = {'consistencygroups': []}
        self.volume_sdk_client.get.side_effect = [
            fakes.FakeResponse(status_code=http.HTTPStatus.NOT_FOUND),
            fakes.FakeResponse(data=data),
        ]
        self.assertRaises(
            osc_lib_exceptions.NotFound,
            volume.find_consistency_group,
            self.volume_sdk_client,
            'invalid-cg',
        )

    def test_find_consistency_group_by_name_duplicate(self):
        cg_name = 'name-' + uuid.uuid4().hex
        data = {
            'consistencygroups': [
                {
                    'id': uuid.uuid4().hex,
                    'name': cg_name,
                },
                {
                    'id': uuid.uuid4().hex,
                    'name': cg_name,
                },
            ],
        }
        self.volume_sdk_client.get.side_effect = [
            fakes.FakeResponse(status_code=http.HTTPStatus.NOT_FOUND),
            fakes.FakeResponse(data=data),
        ]

        self.assertRaises(
            osc_lib_exceptions.NotFound,
            volume.find_consistency_group,
            self.volume_sdk_client,
            cg_name,
        )
